/////////////// Initializations for battery model ////////////////////

// initializes the matrix with random connections
inline InitAllRandom(){
		byte i, j, k;
		mtype b;
		Cell c;
		CellRef cr; 

		// set random connection values for group membership and cell connectivity
		for(i : 0 .. ROWS - 1) { 
			for (j : 0 .. COLS - 1) {
				select (b: idle .. disc); 
				c.conn = b;
				select (k: 0 .. MAXGRNUM -1 ); 
				c.group = k;
				AssignCell(cells[i].cell[j], c);

				AddCellRef(k, i, j);
			}
		}

		// set random connectivity for groups
		for(i : 0 .. MAXGRNUM - 1) { 
			select (b: idle .. disc); 
			groups[i].conn = b;
			}

		// using up the whole space
		groupNum = MAXGRNUM;
	}

// all cells are in one group linearly connected to one another
inline InitSingleGroup() {
	byte i, j;
	
	assert(MAXGRSIZE >= ROWS*COLS);

	CleanCellRefs();

	for(i : 0 .. ROWS - 1) { 
		for (j : 0 .. COLS - 1) {
			cells[i].cell[j].group = 0;

			AddCellRef(0, i, j);
		}
	}

	groupNum = 1;
}
	
// put cells into parallel groups composed by rows. Each group size of groupsize.
inline InitParallelGroups(groupSize) {
	byte i, j, group = 0, cellCount = 0;

	assert(ROWS*COLS <= MAXGRNUM * groupSize);
	assert(groupSize <= MAXGRSIZE);

	CleanCellRefs();

	#ifdef COLSTOROWS
	for (j : 0 .. COLS - 1) {
		for(i : 0 .. ROWS - 1) { 
	#endif
	#ifdef ROWSTOCOLS
	for(i : 0 .. ROWS - 1) { 
		for (j : 0 .. COLS - 1) {
	#endif
			if
			:: cellCount == groupSize -> 
				cellCount = 0;
				group++;
			:: else skip;
			fi;

			cells[i].cell[j].group = group;
			AddCellRef(group, i, j);

			cellCount++;
		}
	}

	// can set groupNum aggressively (if groupcount > CELLSREQD) or safely (always drop the last group)
	if
	:: cellCount == 0 -> 
		groupNum = group;
	:: cellCount >= 1 -> // filled up the last group
		groupNum = group + 1;
	fi; 
}

// attribute initializations
inline InitGroupConn(x) { 
	byte i;
	for(i : 0 .. MAXGRNUM - 1) { 
		groups[i].conn = x;
		}
	}

inline InitCellConn(x) { 
	byte i, j;
	for(i : 0 .. ROWS - 1) { 
		for (j : 0 .. COLS - 1) {
			cells[i].cell[j].conn = x;
		}
	}
	}


inline InitCellCharge(x) { 
	byte i, j;
	for(i : 0 .. ROWS - 1) { 
		for (j : 0 .. COLS - 1) {
			cells[i].cell[j].charge = x;
		}
	}
}
