////////////////////// Utilities for battery model in Promela /////////////////////////////
//// To be included with the model itself ////

inline SwapInt(arg1, arg2) { 
	int _t = arg1;
	arg1 = arg2;
	arg2 = _t;

	_t = 0;
}

inline SwapShort(arg1, arg2) { 
	short _t = arg1;
	arg1 = arg2;
	arg2 = _t;

	_t = 0;
}

// a cell copying operation
inline AssignCell(arg1, arg2) { 
	arg1.group = arg2.group;
	arg1.conn = arg2.conn;
	arg1.charge = arg2.charge;
}

// a cell ref copying operation
inline AssignCellRef(cellref1, cellref2) { 
	cellref1.x = cellref2.x;
	cellref1.y = cellref2.y;
}

// a cellref adding operation
inline AddCellRef(groupNumber, cellx, celly) { 
	groups[groupNumber].cellref[groups[groupNumber].size].x = cellx;
	groups[groupNumber].cellref[groups[groupNumber].size].y = celly;
	groups[groupNumber].size++;
}

// calculates the 1-norm distance between cells
inline AssignDistance(distVar, cellref1, cellref2) { 
	if
		:: cellref1.x >= cellref2.x -> 
			distVar = cellref1.x - cellref2.x;
		:: else 
			distVar = cellref2.x - cellref1.x;
	fi;

	if
		:: cellref1.y >= cellref2.y -> 
			distVar = distVar + cellref1.y - cellref2.y;
		:: else 
			distVar = distVar + cellref2.y - cellref1.y;
	fi;
}

// reset cell state
inline ResetCell(_c) { 
	_c.group = 0; 
	_c.conn = idle;
	_c.charge = false;
	}

// reset cell ref state
inline ResetCellRef(_cr) { 
	_cr.x = 0;
	_cr.y = 0;
	}

// reset group's input and output
inline ResetGroupInputOutput(_g) { 
	groups[_g].input.x = 0;
	groups[_g].input.y = 0;
	groups[_g].output.x = 0;
	groups[_g].output.y = 0;
}

// purges all cell references from groups
inline CleanCellRefs() { 
	byte _i, _j; 
	for (_i : 0 .. MAXGRNUM - 1) { 
		groups[_i].size = 0;
		groups[_i].input.x = 0;
		groups[_i].input.y = 0;
		groups[_i].output.x = 0;
		groups[_i].output.y = 0;

		for(_j: 0 .. MAXGRSIZE-1) { 
			groups[_i].cellref[_j].x = 0;
			groups[_i].cellref[_j].y = 0;
		}
	} 

	groupNum = 0;

	// not necessary because group formation already changes groups
	/*for (_i: 0 .. ROWS - 1) {
		for (_j: 0 .. COLS - 1) {
		cells[_i].cell[_j].group = 0;
		}
	} */

	_i = 0;
	_j = 0;
}

// printout cell groups and connectivity 
inline Print(){  
	Cell _c;
        byte _i, _j; 

        for(_i : 0 .. ROWS - 1) {  
		for (_j : 0 .. COLS - 1) { 
			AssignCell(_c, cells[_i].cell[_j]);
			//G Group number C cell connectivity status GC group connectivity status  D discharges till rest R rests till charge
			printf("g%dc%d", _c.group, _c.charge); 
			
		} 
		printf("\n"); 

		for (_j : 0 .. COLS - 1) { 
			printf("      "); 
		} 
		printf("\n"); 
	} 

	for(_i: 0 .. groupNum - 1){ 
		printf("gr%d: in %d%d, out %d%d\n", _i, groups[_i].input.x, groups[_i].input.y, groups[_i].output.x, groups[_i].output.y);
	}

	printf("\n"); 

	ResetCell(_c);
	_i = 0; 
	_j = 0; 
} 
