/* A model for resource allocation with global scheduling and abstracted notion of time (only the sequence of scheduling points is modeled)*/
/* This program models scheduling across several processors */
// # of priority levels is assumed to be equal to the # of threads

#define procnum 8 
#define cpunum 1

#include "sched-util.pml"

// thread/task static parameters from aadl
int wcet[procnum];// task worst case execution time
int period[procnum]; // task period
int deadline[procnum]; // task deadline 
int prior[procnum]; // task priority level
int initialArrival[procnum];//  the first arrival time

// timing properties of a job
int jobLastArrival[procnum]; // last absolute time when a job arrived, -1 if never arrived
int jobLastActivation[procnum]; // absolute time when the current job was last dispatched to a processor, -1 if hasn't arrived or been dispatched yet
int jobDeadline[procnum]; // absolute time of the current job's deadline, -1 if hasn't arrived yet
int jobWork[procnum]; // the current job's work done towards completion, -1 if hasn't arrived yet

// execution sequencing
chan queues[procnum] = [procnum] of {byte}; // the priority queue
bool running[procnum]; // tracks which threads are running on processors
bool terminateExecution; // a flag set when a manager detects a termination; disables future arrivals
bool threadTurn[procnum];// true if a thread can take an action valid on the time horizon
bool managerTurn; // a flag that is set when the priority queue is modified. Marks a scheduling point

// scheduler parameters
bool enableEDF;
int lastArrivedJob; // set to the ID of the last arrived job on the arrival scheduling point; otherwise -1

// time horizon 
int clock;
bool det[procnum]; // a manager tells the threads whether their next event is deterministic (i.e. to advance the clock or not)

// TODO how to handle the ordering of events that arrive at the same instant. Right now: considering all orderings

init {
	atomic {
		printf("Initializing\n");
		int i;

		/*wcet[0] = 4;
		wcet[1] = 4;
		wcet[2] = 4;

		period[0] = 7;
		period[1] = 8;
		period[2] = 8;

		deadline[0] = 7;
		deadline[1] = 6;
		deadline[2] = 8;

		// if EDF, initialize proportionally to deadlines
		prior[0] = 1;
		prior[1] = 2;
		prior[2] = 0;

		bool fixedPhase = false;
		enableEDF = true;*/
		
		/*
period[0] = 100;
period[1] = 150;
period[2] = 200;

deadline[0] = 100;
deadline[1] = 90;
deadline[2] = 200;

wcet[0] = 60;
wcet[1] = 15;
wcet[2] = 20;

// if EDF, initialize proportionally to deadlines
prior[0] = 2;
prior[1] = 1;
prior[2] = 0;

enableEDF = true;
bool fixedPhase = true;*/

		enableEDF = false;
		bool fixedPhase = true;

		/*
		for (i : 0 .. procnum -1) {

			period[i] = 10 + (procnum + i)*5;	
			deadline[i] = 10 + (procnum + i)*5;	
			wcet[i] = 1;

			prior[i] = procnum - 1 - i;
		}*/

		for (i : 0 .. procnum -1) {

			if 
				:: i == 0 -> 
					period[0] = 2;
					deadline[0] = 2;
				:: else -> 
					period[i] = period[i-1]*2;
					deadline[i] = deadline[i-1]*2;
			fi;
			wcet[i] = 1;
			prior[i] = procnum - 1 - i;
		}

// conforms to EDF 
/*period[0] = 100;
period[1] = 150;
period[2] = 200;

deadline[0] = 100;
deadline[1] = 90;
deadline[2] = 200;

wcet[0] = 10;
wcet[1] = 15;
wcet[2] = 20;

// if EDF, initialize proportionally to deadlines
prior[0] = 2;
prior[1] = 1;
prior[2] = 0;*/


		// explore initial phasings
		if 
		:: else skip; 
		:: !fixedPhase -> 
			printf("Selected arrival phasing:");
			int arrival;
			for(i: 0 .. procnum - 1) {
				select(arrival: 0 .. period[i] - 1);	
				initialArrival[i] = arrival;
				printf("%d", initialArrival[i]);
			}	
			printf("\n");
		fi;

		// setting initial values for runtime variables
		for(i: 0 .. procnum - 1) {
			running[i] = false;
			jobLastArrival[i] = -1;
			jobLastActivation[i] = -1;
			jobDeadline[i] = -1;
			jobWork[i] = -1;

			threadTurn[i] = false;
			det[i] = false;
		}

		// sequencing setup
		managerTurn = true; // manager goes first
		terminateExecution = false; 
		lastArrivedJob = -1; 
		clock = 0;
	
		run Manager();
		int id; 
		for (id: 0 .. procnum - 1) {
			run Thread(id);
		}

		printf("Initialization successful\n");
	}
}

proctype Thread(byte ID) {
	do

	// arrival of a job. Note: using atomic here because the select statement contains a non-deterministic loop
	// while a model is terminating a thread cannot arrive again	
	:: atomic { threadTurn[ID] && !running[ID] && !managerTurn && !terminateExecution -> 
		printf("Arrival for thread %d, det: %d\n", ID, det[ID]);
		queues[prior[ID]]!ID;

		if 
			:: det[ID] && jobLastArrival[ID] != -1 -> 
				clock = jobLastArrival[ID] + period[ID]; // deterministic advancement for a returning thread
			:: det[ID] && jobLastArrival[ID] == -1 -> 
				clock = initialArrival[ID]; // deterministic advancement for the first arrival
				printf("Advanced clock to %d\n", clock);
			:: !det[ID] -> // arrivals cannot be non-deterministic
				printf("Exception - this shouldn have happened!\n");
				assert(false);
		:: else skip;
			fi;

		// note: there cannot be a case where a non-first arrival is non-deterministic
		jobLastArrival[ID] = clock;
		jobDeadline[ID] = clock + deadline[ID];
		jobWork[ID] = 0;
		// not modifying jobLastActivation because it hasn't been dispatched

		managerTurn = true; 
		lastArrivedJob = ID; // for EDF
		printf("Status: clock: %d, jobLastArrival:%d, jobLastActivation:%d, jobDeadline:%d, jobWork:%d\n", clock, jobLastArrival[ID], jobLastActivation[ID], jobDeadline[ID], jobWork[ID]);
	}

	// termination of a job
	::d_step { threadTurn[ID] && running[ID] && !managerTurn ->
		printf("Terminating job for thread %d, det: %d\n", ID, det[ID]);

		printf("Status pre thread %d: clock: %d, jobLastArrival:%d, jobLastActivation:%d, jobDeadline:%d, jobWork:%d\n", ID, clock, jobLastArrival[ID], jobLastActivation[ID], jobDeadline[ID], jobWork[ID]);

		queues[prior[ID]]??eval(ID); // may not be the first ID in the queue because of several processors
		running[ID] = false; // doesn't need to record its progress after terminating
		if 
			:: det[ID] -> 
				clock = jobLastActivation[ID] + wcet[ID] - jobWork[ID];
			:: else skip;
		fi;

		if
			:: clock > jobDeadline[ID] ->
				printf("DEADLINE MISSED by %d\n", clock-jobDeadline[ID]);
				assert(false);
			:: else skip;
		fi;

		// purging the state
		jobDeadline[ID] = -1;
		jobLastActivation[ID] = -1; 
		jobWork[ID] = -1; // this has to be after the clock update
		// not modifying jobLastArrival because it's not a job's parameter

		managerTurn = true; 
		lastArrivedJob = -1; // for EDF

		printf("Status post thread %d: clock: %d, jobLastArrival:%d, jobLastActivation:%d, jobDeadline:%d, jobWork:%d\n", ID, clock, jobLastArrival[ID], jobLastActivation[ID], jobDeadline[ID], jobWork[ID]);
	}

	od
}

// does all the dirty work
proctype Manager() {
	do
		::d_step { managerTurn -> 
			printf("\n");
		
			// rewind clock variables
			rewindClocks();

			// run the scheduler
			if
			:: enableEDF ->  EDFScheduler();
			:: else skip;
			fi;

			// take threads off processors
			undispatch();

			// redispatch threads on processors 
			dispatchPriority();

			// calculate the scheduling points
			calculateTimeHorizon();

			// activate threads
			managerTurn = false;

			assert(clock < 10000);
		}
	od
}

// thread 0 can be pre-empted only when its deadline is not further 
ltl p0 { [] true }

/*
ltl p1 {
	[](	
		(running[1] && !running[0] && queues[prior[0]]?[0] && !managerTurn-> deadline[1] <= deadline[0]) 
	)
}
ltl p2 {
	[](	
		(running[0] && !running[1] && queues[prior[1]]?[1] && !managerTurn -> deadline[0] <= deadline[1]) 
	)
}ltl p3 {
	[](	
		(running[0] && !running[2] && queues[prior[2]]?[2]  && !managerTurn-> deadline[0] <= deadline[2])
	)
}ltl p4 {
	[](	
		(running[2] && !running[0] && queues[prior[0]]?[0]  && !managerTurn-> deadline[2] <= deadline[0]) 
	)
}ltl p5 {
	[](	
		(running[1] && !running[2] && queues[prior[2]]?[2]  && !managerTurn-> deadline[1] <= deadline[2])
	)
}ltl p6 {
	[](	
		(running[2] && !running[1] && queues[prior[1]]?[1]  && !managerTurn-> deadline[2] <= deadline[1])
	)
}

ltl p100 {
	[](  
	(	
		(running[1] && !running[0] && queues[prior[0]]?[0] && !managerTurn-> deadline[1] <= deadline[0]) 
	)
	 && (	
		(running[0] && !running[1] && queues[prior[1]]?[1] && !managerTurn -> deadline[0] <= deadline[1]) 
	)
	&& (	
		(running[0] && !running[2] && queues[prior[2]]?[2]  && !managerTurn-> deadline[0] <= deadline[2])
	)
	&& (	
		(running[2] && !running[0] && queues[prior[0]]?[0]  && !managerTurn-> deadline[2] <= deadline[0]) 
	)
	&& (	
		(running[1] && !running[2] && queues[prior[2]]?[2]  && !managerTurn-> deadline[1] <= deadline[2])
	)
	&& (	
		(running[2] && !running[1] && queues[prior[1]]?[1]  && !managerTurn-> deadline[2] <= deadline[1])
	)
	)
}*/
